from typing import List


class CSETicker(object):
    def __init__(
        self,
        updated_at: dict,
        metatdata: dict,
        quote: dict,
        depth_by_order: List[dict],
        depth_by_price: List[dict],
        ticker: dict,
        trades: List[dict],
    ):
        self.updated_at = updated_at
        self.metatdata = metatdata
        self.quote = quote
        self.depth_by_order = depth_by_order
        self.depth_by_price = depth_by_price
        self.ticker = ticker
        self.trades = trades


class CSESedarFilings(object):
    def __init__(self, categories: dict, list_items: List[dict]):
        self.categories = categories
        self.list_items = list_items



from cad_tickers.exchanges.cse import get_cse_tickers_df, get_all_cse_tickers
from cad_tickers.exchanges.tsx import get_all_tsx_tickers, get_all_tickers_data
import pandas as pd


def mk_full_tickers() -> pd.DataFrame:
    """
      gets all tickers from the cse and tsx/tsxv using
      webmoney as a proxy

    Returns: full ticker df

    """
    cse_df = get_cse_tickers_df()
    cse_tickers = get_all_cse_tickers(cse_df)
    # tsx and tsxv tickers
    tsx_tickers = get_all_tsx_tickers()

    full_tickers = [
        *tsx_tickers,
        *cse_tickers,
    ]
    full_df = get_all_tickers_data(tickers=full_tickers)
    return full_df


if __name__ == "__main__":
    full_df = mk_full_tickers()
    full_df.to_csv("testing.csv")



import pandas as pd
import bs4
import requests
import os
from typing import List


def transform_name_to_slug(raw_ticker: str) -> str:
    """
    Parameters:
      raw_ticker - cse ticker to be converted to slug
    Returns:
      transformed - raw_ticker
    """
    transformed = raw_ticker.lower().replace(".", "").replace(" ", "-")
    return transformed


def parse_description_tags(description_tags: List[bs4.element.Tag]) -> str:
    """
    Parameters:
      description_tags - html tags from webpage, usually p tag containing description
    Returns:
      description - description for ticker
    """
    if len(description_tags) > 0:
        description_tag = description_tags[0]
        # grab contents from desscription tag
        if len(description_tag.contents) == 0:
            return ""
        else:
            return description_tag.contents[0]
    # company does not have profile
    else:
        return ""


def make_cse_path(raw_ticker: str, raw_industry: str) -> str:
    """makes slug for ticker for the cse

    Parameters:
      raw_ticker - cse ticker from xlsx sheet
      raw_industry - verbatim industry from ticker, not slugified
    Returns:
      description - url for cse files for download
    """
    if pd.isna(raw_industry):
        return ""
    # verify raw_industry is in industry do later
    cse_industries = [
        "Industry",
        "Mining",
        "Diversified Industries",
        "Life Sciences",
        "Oil and Gas",
        "Technology",
        "CleanTech",
    ]

    base_cse_url = "https://thecse.com/en/listings"
    industry = raw_industry.lower().replace(" ", "-")
    ticker = transform_name_to_slug(raw_ticker)
    url = f"{base_cse_url}/{industry}/{ticker}"
    return url


def read_df_from_file(file_path: str) -> pd.DataFrame:
    """
    Parameters:
      file_path - path to data
    Returns:
      df - excel sheet dataframe
    """
    try:
        df = pd.read_excel(file_path)
    except Exception:
        df = pd.read_csv(file_path)
    return df


def tickers_to_ytickers(tsx_path: str, cse_path: str) -> List[str]:
    """
    Parameters:
      tsx_path - path to clean tsx file
      cse_path - path to clean cse file
    Returns:
      ytickers - list of tickers
    """
    # grab tsx data
    tsx_df = read_df_from_file(tsx_path)
    tsx_df = tsx_df[["Ex.", "Ticker"]]
    ytickers_series = tsx_df.apply(tsx_ticker_to_yahoo, axis=1)
    ytickers_series = ytickers_series.drop_duplicates(keep="last")
    tsx_tickers = ytickers_series.tolist()

    cse_df = read_df_from_file(cse_path)
    cse_df = cse_df[["Symbol"]]
    ytickers_series = cse_df.apply(cse_ticker_to_yahoo, axis=1)
    ytickers_series = ytickers_series.drop_duplicates(keep="last")
    cse_tickers = ytickers_series.tolist()

    ytickers = [*tsx_tickers, *cse_tickers]
    return ytickers


def cse_ticker_to_yahoo(row: pd.Series) -> str:
    """
    Parameters:
      row - series from cse dataframe
    Returns:
      ticker - yahoo ticker for cse
    """
    ticker = row["Symbol"]
    return f"{ticker}.CN"


def tsx_ticker_to_yahoo(row: pd.Series) -> str:
    """
    Parameters:
      row - pd.Series
        * Ticker - ticker from pandas dataframe from cad_tickers
        * Ex. - what exchange the ticker is for
    Returns:
      yticker - yahoo finance ticker for tsx
    """
    ticker = row["Ticker"]
    exchange = row["Ex."]
    # 1min, 5min, 15min, 30min, 60min, daily, weekly, monthly
    switcher = {"TSXV": "V", "TSX": "TO"}
    yahoo_ex = switcher.get(exchange, "TSXV")
    return f"{ticker}.{yahoo_ex}"


def cse_ticker_to_webmoney(cse_ticker: str):
    """
    Parameters:
      cse_ticker - cse ticker name
    Returns:
      webmoney_ticker - ticker that can be looked up in webmoney
    """
    return f"{cse_ticker}:CNX"


# make request with query params


# 


import requests
import re
import pandas as pd
from concurrent.futures import ThreadPoolExecutor
from bs4 import BeautifulSoup
from cad_tickers.util import (
    parse_description_tags,
    make_cse_path,
    cse_ticker_to_webmoney,
)


def get_all_cse_tickers(cse_df: pd.DataFrame) -> list:
    """
    Parameters:
        cse_df - cleaned cse dataframe
    Returns:
        webmoney_tickers - list of webmoney cse tickers
    """
    try:
        tickers = cse_df["Symbol"].values.tolist()
        webmoney_tickers = [cse_ticker_to_webmoney(ticker) for ticker in tickers]
        return webmoney_tickers
    except Exception as e:
        print("FAILED TO GRAB TICKER FROM CSE_DF")
        print(e)
        return []


def get_cse_files(filename: str = "cse.xlsx", filetype: str = "xlsx") -> str:
    """Gets excel spreadsheet from api.tsx using requests

    Parameters:
      filename: Name of the file to be saved
      filetype: Save as pdf or xlsx

    Returns:
      filePath returns path to file

    See ://stackoverflow.com/questions/13567507/passing-csrftoken-with-python-requests

    """
    # TODO force it to be pdf or xlsx
    # https://www.thecse.com/export-listings/xlsx?f={}
    # https://www.thecse.com/export-listings/pdf?f={}
    URL = f"https://www.thecse.com/export-listings/{filetype}?f=" + r"{}"
    r = requests.get(URL)
    responseHeaders = r.headers
    if "text/html" in responseHeaders["Content-Type"]:
        return None
    elif any(
        re.findall(
            r"application/vnd.ms-excel|application/pdf|xlsx",
            responseHeaders["Content-Type"],
            re.IGNORECASE,
        )
    ):
        respData = r.content
        filepath = f"{filename}"
        with open(filepath, "wb") as s:
            s.write(respData)
        return filepath
    else:
        return None


def clean_cse_data(raw_df: pd.DataFrame) -> pd.DataFrame:
    """Removes bad data from cse dataframe.

    Parameters:
      raw_df
        Dataframe with mostly unnamed columns from pandas df import

        =============  ====================================================================
        CSE Listings   Label for Company data
        Unnamed: 1     Listing symbol from the cse exchange needs a mapper to yahoo finance
        Unnamed: 2     Enum of industry including Mining
        Unnamed: 3     Enum such as CSE Composite
        Unnamed: 4     Enum such as CSE Composite
        Unnamed: 5     Usually CAD
        Unnamed: 6     empty (pandas import error, dropped)
        Unnamed: 7     Date when trading started
        =============  ====================================================================

    Returns:
      clean_df
        Dataframe with bad data removed

        ==========  ====================================================================
        Company     Full name of the company
        Symbol      Listing symbol from the cse exchange needs a mapper to yahoo finance
        Industry    Enum of industry including Mining
        Identifier  Broad category (US Cannabis)
        Indices     Enum such as CSE Composite
        Currency    Usually CAD
        Trading     Date when trading started
        urls        url to listing on cse website
        ==========  ====================================================================
    """
    # drop all nans
    df = raw_df
    df = df.dropna(axis=0, how="all")
    # drop empty column
    df = df.drop(df.columns[6], axis=1)
    # grab logically column names from first row
    column_labels = df.iloc[1, :].values.tolist()
    df.columns = column_labels
    # dropping title row and column titles row
    df = df.drop([df.index[0], df.index[1]])
    df = df.reset_index(drop=True)
    return df


def get_cse_tickers_df() -> pd.DataFrame:
    """Grab cse dataframe from exported xlsx sheet

    Returns:
      clean_df
        Dataframe with with randomly selected values. Data columns are as follows:

        ==========  ====================================================================
        Company     Full name of the company
        Symbol      Listing symbol from the cse exchange needs a mapper to yahoo finance
        Industry    Enum of industry including Mining
        Identifier  Broad category (US Cannabis)
        Indices     Enum such as CSE Composite
        Currency    Usually CAD
        Trading     Date when trading started
        urls        url to listing on cse website
        ==========  ====================================================================
    """
    URL = f"https://www.thecse.com/export-listings/xlsx?f=" + r"{}"
    r = requests.get(URL)
    responseHeaders = r.headers
    if "text/html" in responseHeaders["Content-Type"]:
        return None
    elif any(
        re.findall(
            r"application/vnd.ms-excel|application/pdf|xlsx",
            responseHeaders["Content-Type"],
            re.IGNORECASE,
        )
    ):
        respData = r.content
        df = pd.read_excel(respData)
        clean_df = clean_cse_data(df)
        # add urls
        clean_df["urls"] = clean_df.apply(
            lambda x: make_cse_path(x["Company"], x["Industry"]), axis=1
        )
        return clean_df
    else:
        return None


def get_description_for_url(url: str) -> str:
    """
    Parameters:
      url - link to ticker can be empty string
    Returns:
      description - details of what the ticker does, can be empty string
    """
    if url == "":
        return ""
    r = requests.get(url)
    html_content = r.text
    soup = BeautifulSoup(html_content, "lxml")
    description_selector = "#block-system-main div.company-description > p"
    description_tags = soup.select(description_selector)
    return parse_description_tags(description_tags)


def add_descriptions_to_df(df: pd.DataFrame, max_workers: int = 16) -> pd.DataFrame:
    """
    Parameters:
      clean_df
        Dataframe with with randomly selected values. Data columns are as follows:

        ==========  ====================================================================
        Company     Full name of the company
        Symbol      Listing symbol from the cse exchange needs a mapper to yahoo finance
        Industry    Enum of industry including Mining
        Identifier  Broad category (US Cannabis)
        Indices     Enum such as CSE Composite
        Currency    Usually CAD
        Trading     Date when trading started
        urls        url to listing on cse website
        ==========  ====================================================================

      max_workers
        maximum number of thread workers to have

    Returns:
      df
        Dataframe descriptions in every column if valid

        ===========  ====================================================================
        Company      Full name of the company
        Symbol       Listing symbol from the cse exchange needs a mapper to yahoo finance
        Industry     Enum of industry including Mining
        Identifier   Broad category (US Cannabis)
        Indices      Enum such as CSE Composite
        Currency     Usually CAD
        Trading      Date when trading started
        urls         url to listing on cse website
        description  cse description scrapped from website
        ===========  ====================================================================

    """
    urls = df["urls"].tolist()
    with ThreadPoolExecutor(max_workers=max_workers) as tpe:
        iterables = tpe.map(get_description_for_url, urls)
        descriptions = list(iterables)
    df["description"] = descriptions
    return df


if __name__ == "__main__":
    from datetime import datetime
    import argparse

    start_time = datetime.now()
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-f", "--file", help="file name (default: %(default)s)'", default="cse.xlsx"
    )
    parser.add_argument(
        "-t",
        "--type",
        default="xlsx",
        const="xlsx",
        nargs="?",
        choices=("xlsx", "pdf"),
        help="xlsx or pdf (default: %(default)s)",
    )
    cse_df = get_cse_tickers_df()
    cse_df.to_csv("cse.csv")
    # args = parser.parse_args()
    # cse_df = get_cse_tickers_df()
    # df = add_descriptions_to_df(cse_df)
    # end_time = datetime.now()
    # get_cse_files(args.file, args.type)
    # print(df)



import requests
import pandas as pd
from datetime import datetime
from concurrent.futures import ThreadPoolExecutor
from cad_tickers.exchanges.tsx.get_ticker_data import get_ticker_data

# since tsx lacks documentation
# unlikely that I can figure out what the other endpoints do
tsx_json_url = "https://www.tsx.com/json/company-directory/search"


def get_all_tsx_tickers() -> list:
    """
    Returns:
        all_tsx - list of tickers for toronto stock exchange
    """
    tsx_tickers = get_tsx_tickers()
    tsxv_tickers = get_tsx_tickers("tsxv")
    all_tsx = [*tsx_tickers, *tsxv_tickers]
    return all_tsx


def get_tsx_tickers(exchange="tsx") -> list:
    """
    Parameters:
        exchange - tsx or tsxv
    Returns:
        symbol_list - list of symbols for exchange
    """
    if exchange not in ["tsxv", "tsx"]:
        raise Exception(f"Expect {exchange} as tsxv or tsx.")
    # get all tickers from endpoint
    endpoint = f"{tsx_json_url}/{exchange}/.*"
    r = requests.get(endpoint)
    data = r.json()
    results = data.get("results", [])
    # error out so I can fix the errors
    assert len(results) > 0
    symbol_list = []
    for result in results:
        symbol = result.get("symbol")
        symbol_list.append(symbol)
    num_results = data.get("length")
    assert num_results == len(symbol_list)
    return symbol_list


def get_all_tickers_data(tickers=None, max_workers: int = 16):
    """
    Parameters:
        max_workers - number of workers for ThreadPoolExecutor
    Returns:
        ticker_df - pd.DataFrame with columns of :ref:`Quote By Symbol <quote_by_symbol_query>`
    """
    if tickers == None:
        tickers = get_all_tsx_tickers()
    ticker_data = []
    with ThreadPoolExecutor(max_workers=max_workers) as tpe:
        iterables = tpe.map(get_ticker_data, tickers)
        ticker_data = list(iterables)

    # using list comprehension
    # to remove None values in list
    ticker_data = [i for i in ticker_data if i]

    ticker_df = pd.DataFrame(ticker_data)
    return ticker_df


if __name__ == "__main__":
    startTime = datetime.now()
    data = get_all_tickers_data()
    data.to_csv("tsx_data.csv")
    print(data)
    print(datetime.now() - startTime)


import requests
import bs4
import pandas as pd
from typing import List, Union, Tuple
from concurrent.futures import ThreadPoolExecutor


def scrap_news_for_ticker(ticker: str) -> List[dict]:
    """Extracts webpage data from a ticker

    TODO add a delay

    Parameters:
      ticker - yahoo finance ticker
    Returns:
      news_data - list of dicts extracted from webpage
        * source - str
        * link_href - link from post (can be relative or absolute)
        * link_text - description for link
        * ticker - reference to original ticker

    """
    try:
        yahoo_base_url = "https://finance.yahoo.com"
        news_items, html_content = get_ynews_for_ticker(ticker, yahoo_base_url)
        news_data = []
        for news_item in news_items:
            # remove comments
            for comment in news_item(text=lambda text: isinstance(text, bs4.Comment)):
                comment.extract()
            # grab description header, stuff next too image
            news_content = news_item.find(
                "div", {"class": "Ov(h) Pend(44px) Pstart(25px)"}
            )
            if news_content is None:
                # try again with different class
                news_content = news_item.find("div", {"class": "Ov(h)"})
            source = find_news_source(news_content)
            link_href, link_text = find_news_link_and_text(news_content)
            news_data.append(
                {
                    "source": source,
                    "link_href": link_href,
                    "link_text": link_text,
                    "ticker": ticker,
                }
            )
        return news_data
    except Exception as e:
        print(e)
        return []


def get_ynews_for_ticker(
    ticker: str, yahoo_base_url="https://finance.yahoo.com"
) -> List[bs4.element.Tag]:
    """Returns initial news items fetched from yahoo when loading quote page.
    Since yahoo has lazy loading, not all items are returned. Seems like ads are not loaded
    because of lazy loading.

      Parameters:
        ticker - yahoo formatted ticker str
        yahoo_base_url - optional parameter that is the base of the request

      Returns:
        news_items - list of key html content for the news item
    """
    yahoo_qurl = f"{yahoo_base_url}/quote/{ticker}?p={ticker}"
    r = requests.get(yahoo_qurl)
    html_content = r.text
    soup = bs4.BeautifulSoup(html_content, "lxml")
    news_items = soup.find_all(
        "li", {"class": "js-stream-content Pos(r)"}
    )  # This will return a list of all line items in the markup.
    return news_items, html_content


def find_news_link_and_text(news_content: bs4.element.Tag) -> Tuple[str, str]:
    """Finds news link from news_content.
       Assumes comments are deleted from the yahoo finance news items

    Parameters:
      news_content - html based data for the news article

    Returns:
      link_href - link in html markup
      link_text - link text in html markup
    """
    if news_content is None:
        return "", ""
    link_tag = news_content.find("a")
    link_href = link_tag.get("href")
    link_text = link_tag.text
    return link_href, link_text


def find_news_source(news_content: bs4.element.Tag) -> Union[None, str]:
    """Utility function to verify news format from yahoo has not changed

    when grabbing data from yahoo with requests, it seems date is not returned.

    Parameters:
      news_content: html based data for the news article

    Returns:
      source - publisher of article

    wrapper div around content - such as - CNW Group 2 days ago
    """
    if news_content is None:
        return
    # wrapper div around content - such as - CNW Group 2 days ago
    wrapper_div = news_content.find(
        "div", {"class": "C(#959595) Fz(11px) D(ib) Mb(6px)"}
    )
    source = wrapper_div.text
    return source


if __name__ == "__main__":
    scrap_news_for_ticker("IP.CN")



import requests
import pandas as pd
from datetime import datetime


def get_halts_resumption() -> pd.DataFrame:
    """Gets the latest 25 halts from the iiroc

    Returns:
      halt_df
        Dataframe with bad data removed

        ==========  ====================================================================
        Halts       Details of halts
        Listing     Extracted ticker from halt
        ==========  ====================================================================
    """
    now = datetime.now()
    year = now.year
    # get year
    iiroc_url = f"https://iiroc.mediaroom.com/index.php?year={year}"
    r = requests.get(iiroc_url)
    html_data = r.text
    halts_df = pd.read_html(html_data, attrs={"class": "spintable"})
    halt_df = halts_df[0]
    halt_df.columns = ["Halts"]
    halt_df["Listing"] = halt_df["Halts"].str.split().str[-1]
    halt_df = halt_df.drop(halt_df.tail(1).index)
    return halt_df
    # extract last word as
    # #wd_printable_content > div.wd_newsfeed_releases > div > table


if __name__ == "__main__":
    get_halts_resumption()



"""

Stock News
----------

Downloads stock ticker news from yahoo such as from https://ca.finance.yahoo.com/quote/IP.CN/
 

IIROC - Investment Industry Regulatory Organization of Canada (IIROC) contains 
information about stock halts

https://www.iiroc.ca/Pages/default.aspx
"""

from cad_tickers.news.iiroc_halts import *
from cad_tickers.news.stock_news import *


import requests
import bs4
from bs4 import BeautifulSoup   

def extract_article(article_url: str)-> bs4.element.Tag:
    """Extracts data from given ceo news url"""
    r = requests.get(article_url)
    if r == None:
        print('No data returned from the url')
        print('need more delay???')
    data = r.text
    soup = BeautifulSoup(data, features="html.parser")
    try:
        scripts = [x.extract() for x in soup.findAll('script')]
    except AttributeError as e:
        print('No Scripts to Extract from article')
    # print(soup.prettify())
    article = soup.find(attrs={'class': 'article-body article'})
    # Use this as a example
    # remove image tags
    if article == None:
        article = soup.find(id='article')
    try:
        image_text = article.findAll(lambda tag : tag.name == 'span' and 'Click Image To View Full Size' in tag.text)
        [x.extract() for x in image_text]
    except AttributeError as e:
        print('No Click Image to View Full Size text')
    try:
        images = [x.extract() for x in soup.findAll('img')]
    except AttributeError as e:
        print('No Images in news report')
    return article

def save_bs4_tag(tag: bs4.element.Tag, file_name: str=''):
    """Save bs4 tag to file"""
    with open(file_name, "w", encoding='utf-8') as f:
        f.write(tag.text)

if __name__ == '__main__':
    pass


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cad_tickers',
 'cad_tickers.exchanges',
 'cad_tickers.exchanges.tsx',
 'cad_tickers.news',
 'cad_tickers.news.ceo',
 'cad_tickers.sedar',
 'cad_tickers.util']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.1,<5.0.0',
 'coverage>=5.2,<6.0',
 'lxml>=4.5.2,<5.0.0',
 'pandas>=1.0.5,<2.0.0',
 'requests>=2.24.0,<3.0.0',
 'xlrd>=1.2.0,<2.0.0']

setup_kwargs = {
    'name': 'cad-tickers',
    'version': '1.4.6',
    'description': 'Various Stock Utilties Created by me',
    'long_description': '[![PyPI version](https://badge.fury.io/py/cad-tickers.svg)](https://badge.fury.io/py/cad-tickers) [![Downloads](https://pepy.tech/badge/cad-tickers)](https://pepy.tech/project/cad-tickers) [![Documentation Status](https://readthedocs.org/projects/cad-tickers/badge/?version=latest)](https://cad-tickers.readthedocs.io/en/latest/?badge=latest) [![codecov](https://codecov.io/gh/FriendlyUser/cad_tickers/branch/master/graph/badge.svg)](https://codecov.io/gh/FriendlyUser/cad_tickers)\n \n## Cad Tickers\nFunction to extract exchange data from the cse and tsx websites and various other data sources. This package is primarily focussed on scrapping data for the canadian stock market.\n\n\nThe entire 0.2.x version of tsx functions are now depricated.\n\n\n### How to run tests\n\n```\npytest\n```\n\n```\n# Needed for readthedocs documentation\npoetry export -f requirements.txt > requirements.txt.\n```\n\n#### Todo\n\n\n#### Donate\n\nIf you would like to motivate me to spend more time improving open source projects please consider donating.\n\n[![Donate with Ethereum](https://en.cryptobadges.io/badge/big/0x9d18acAB9Fe749Cbf899B2FD63Bf25e64829bbF3)](https://en.cryptobadges.io/donate/0x9d18acAB9Fe749Cbf899B2FD63Bf25e64829bbF3)\n\n[![Donate with Bitcoin](https://en.cryptobadges.io/badge/big/1BMWhjCrTE3Dn94oHnrk6XMZAS3hjq3vdD)](https://en.cryptobadges.io/donate/1BMWhjCrTE3Dn94oHnrk6XMZAS3hjq3vdD)\n\n[![paypal](https://www.paypalobjects.com/en_US/i/btn/btn_donateCC_LG.gif)](https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=Z6M6Y83D3URSU&item_name=Motivating+me+to+continue+to+produce+open+source+projects&currency_code=CAD)',
    'author': 'David Li',
    'author_email': 'davidli012345@gmail.com',
    'maintainer': 'David Li',
    'maintainer_email': 'davidli012345@gmail.com',
    'url': 'https://github.com/FriendlyUser/cad_tickers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



import requests
import json
from cad_tickers.exchanges.tsx.gql_data import GQL
from typing import Union


def get_ticker_data(symbol=str) -> Union[dict, None]:
    """
    Parameters:
        symbol - ticker symbol from tsx, no prefix
    Returns:
        dict - :ref:`Quote By Symbol <quote_by_symbol_query>`
    """
    payload = GQL.quote_by_symbol_payload
    payload["variables"]["symbol"] = symbol
    url = "https://app-money.tmx.com/graphql"
    r = requests.post(
        url,
        json=payload,
        headers={
            "authority": "app-money.tmx.com",
            "referer": f"https://money.tmx.com/en/quote/{symbol.upper()}",
            "locale": "en",
        },
    )
    try:
        allData = r.json()
    except json.decoder.JSONDecodeError as error:
        print(error)
        print(f"Failed to decode data for {symbol}")
        print(r)
        return None
    # Check for errors
    try:
        data = allData["data"]["getQuoteBySymbol"]
        return data
    except KeyError as _e:
        print(_e, symbol)
        pass


if __name__ == "__main__":
    import pandas as pd

    data = get_ticker_data("art")
    series = pd.Series(data)



"""

Although some articles are returned from the url https://ceo.ca/@newswire?filters[terms]=APHA&filters[top]=100,

it is adequate to scrap with the api with terms https://ceo.ca/api/get_spiels?channel=%40newswire&until=1593091936374&load_more=top&filters%5Bterms%5D=APHA&filters%5Btop%5D=100&original_scroll_height=1045

best to estimate height difference via scroll height somewhat randomly based on number of entries

469

To scrap.

Call api with terms of interest usually a all caps ticker with name

Then extract the spiels for each and output raw data with cleaned data.

The spiel contain the @newswire just find anything with a @ symbol, should be basic regex.

1. request to ceo.ca with query params including search term, randomized approach
2. use pagination to chain requests - stop after 10 calls and/or timestamp does not change
3. Modify data and return the urls back in an easy to use format, like an list and/or raw data
4. Use external package to analyze articles.
""" 

# 

from cad_tickers.news.ceo.spiel_class import SearchParams
from cad_tickers.news.ceo.utils import params_to_dict, ceo_url, \
 news_link_from_spiel, art_pixel_height

from cad_tickers.news.ceo.main import extract_urls, get_new_items
from cad_tickers.news.ceo.scrap import extract_article, save_bs4_tag


import requests
from datetime import datetime
from cad_tickers.exchanges.tsx.gql_data import GQL
from typing import Union


def get_ticker_filings(
    symbol: str,
    fromDate: str = datetime.today().replace(day=1).strftime("%Y-%m-%d"),
    toDate: str = datetime.today().strftime("%Y-%m-%d"),
    limit: int = 100,
) -> Union[dict, None]:
    """
    Parameters:
        symbol - ticker symbol from tsx, no prefix
        fromDate - start date to grab documents
        toDate - end date to grab documents
        limit - max number of documents to retrieve
    Returns:
        dict - :ref:`Quote By Symbol <quote_by_symbol_query>`
    """
    payload = GQL.get_company_filings_payload
    payload["variables"]["symbol"] = symbol
    payload["variables"]["fromDate"] = fromDate
    payload["variables"]["toDate"] = toDate
    payload["variables"]["limit"] = limit
    url = "https://app-money.tmx.com/graphql"
    r = requests.post(
        url,
        json=payload,
        headers={
            "authority": "app-money.tmx.com",
            "referer": f"https://money.tmx.com/en/quote/{symbol.upper()}",
            "locale": "en",
        },
    )
    allData = r.json()
    try:
        data = allData["data"]
        return data
    except KeyError as _e:
        print(_e, symbol)
        pass


if __name__ == "__main__":
    art = get_ticker_filings(
        "ART", start_date="2015-11-11", end_date="2020-11-11", limit=108
    )
    print(art)



"""
Utilities
----------

Various Utility Functions such as transforming tickers from cse to yahoo
"""
from cad_tickers.util.utils import (
    transform_name_to_slug,
    parse_description_tags,
    make_cse_path,
    read_df_from_file,
    tickers_to_ytickers,
    cse_ticker_to_yahoo,
    tsx_ticker_to_yahoo,
    cse_ticker_to_webmoney
)
